from infi.shell import commandgroup, command, render
from infi.shell.utils import strlist

from .types import Enclosure
from .options import QueryOptions
from .utils import strip_newline_chars
from . import EntityCommands

QUERY_FIELDS           = strlist("id state drives_number                                                      state_description")
INFINIDAT_QUERY_FIELDS = strlist("id state drives_number vendor model firmware last_probe_timestamp probe_ttl state_description")

QUERY_SUBSTITUTES = dict(
    id="index",
    drives_number="drives",
    )

@commandgroup(prefix="enclosure")
class EnclosureCommands(EntityCommands):
    ENTITY_TYPE = Enclosure

    @command(
        description="List all data-drive enclosures",
        options=QueryOptions(fields=QUERY_FIELDS, infinidat_fields=INFINIDAT_QUERY_FIELDS, substitute=QUERY_SUBSTITUTES),
        )
    @render(
        fields=QUERY_FIELDS,
        substitute=QUERY_SUBSTITUTES,
        infinidat=dict(
            fields=INFINIDAT_QUERY_FIELDS,
            ),
        technician=dict(
            fields=INFINIDAT_QUERY_FIELDS,
            ),
        )
    def query(self, _api):
        return self._query(_api, _hook=self._enc_query_hook)

    def _enc_query_hook(self, enclosures):
        for e in enclosures:
            if e.get("state_description"):
                e.state_description = strip_newline_chars(e.state_description)
        return enclosures
