from __future__ import absolute_import

from itertools import chain, izip
import csv
from numbers import Number

from infi.shell.shell import get_active_shell

from .renderer import RendererBase
from .values import ValueRenderer

class CSVRenderer(RendererBase):
    DEFAULT_SEPARATOR = ","
    def __init__(self, *args, **kwargs):
        super(CSVRenderer, self).__init__(*args, **kwargs)
        self._value_renderer = ValueRenderer(empty_value="")
    def _write_row(self, writer, row):
        self._ensure_stream_open()
        writer.writerow(row)
    def _render_impl(self, obj):
        if obj is None:
            return
        if isinstance(obj, (basestring, Number)):
            self._writeln(str(obj))
            return
        elif self._properties.get("individually", False) and isinstance(obj, dict):
            obj = [obj]
        self._render_collection(obj)
    def _render_collection(self, objects):
        it = iter(objects)
        try:
            first_object = next(it)
        except StopIteration:
            return
        field_names, rendered_field_names, _ = self._get_ordered_and_rendered_fields(first_object)
        all_objects = chain([first_object], it)
        writer = csv.DictWriter(self._stream, field_names)
        # Header
        self._write_row(writer, dict(izip(field_names, rendered_field_names)))
        # Rows
        for obj in all_objects:
            self._write_row(writer, { field : self._render_value(obj, field).encode('utf-8', 'ignore') for field in field_names })
    def _render_value(self, obj, field):
        value = obj.get(field, self.ERROR_MISSING_FIELD)
        if value is not self.ERROR_MISSING_FIELD:
            value = self._value_renderer.render(field, value, **self._properties.get("value_properties", {}))
        return value

def toggle_csv_mode(shell=None):
    shell = shell or get_active_shell()
    if not is_csv_mode(shell):
        set_csv_mode(True, shell)
        return True
    set_csv_mode(False, shell)
    return False

def is_csv_mode(shell=None):
    return isinstance((shell or get_active_shell()).get_renderer_override(), CSVRenderer)

def set_csv_mode(enabled, shell=None):
    shell = shell or get_active_shell()
    if enabled:
        shell.set_renderer_override(CSVRenderer())
        shell.disable_stream_encoding()
    else:
        shell.set_renderer_override(None)
        shell.enable_stream_encoding()
