from infi.shell.commands import commandgroup, command
from infi.shell.types import Str
from infi.shell.exceptions import BadArguments

from .types import VolumeOrClone
from . import roles

@commandgroup(prefix="cg")
class ConsGroupCommands(object):
    @command(
        description="Create a consistent set of snapshots of multiple volumes or clones",
        types=(VolumeOrClone.List, Str, Str),
        )
    @roles.INFINIDAT.ADMIN.POOLADMIN
    def ad_hoc_snapshot(self, _api, entities, snap_prefix=None, snap_suffix=None):
        if not snap_suffix and not snap_prefix:
            raise BadArguments("Either snap_prefix, snap_suffix, or both must be specified")
        _api.post(VolumeOrClone.get_uri("group_snapshot"), dict(
            entities=[dict(id=entity.id) for entity in entities],
            snap_prefix=snap_prefix,
            snap_suffix=snap_suffix,
            ))
        return "Consistent snapshots created"
    