from __future__ import print_function, absolute_import

from logging import getLogger
from colorama import Fore, Style
from infi.shell import (
    Shell,
    Executor,
    env,
    runtime,
    )
from infi.shell.utils import ConditionalLineStreamWrapper

from . import debug
from .utils import post_mortem

logger = getLogger("shell")

PRIMARY_COLOR, SECONDARY_COLOR = [Fore.CYAN], [Fore.CYAN, Style.BRIGHT]
CUSTOM_BANNER_PARTS = (
    ("For help on all commands, type '", PRIMARY_COLOR),
    ("help", SECONDARY_COLOR),
    ("'.\n", PRIMARY_COLOR),
    ("For help on a specific command, type '", PRIMARY_COLOR),
    ("help on=vol.create", SECONDARY_COLOR),
    ("' or '", PRIMARY_COLOR),
    ("vol.create?", SECONDARY_COLOR),
    ("'", PRIMARY_COLOR),
    ("\n\n", [Style.RESET_ALL])  # Without applying no-color to the newline chars, the cursor is colored
    )

class ShellMixin(object):
    def _prepare_stream(self, stream):
        grep_pattern, inverted_grep = runtime.options.get("grep"), False
        if not grep_pattern:
            grep_pattern, inverted_grep = runtime.options.get("inverted-grep"), True
        if grep_pattern:
            return ConditionalLineStreamWrapper(stream, lambda line: inverted_grep ^ bool(grep_pattern.search(line)))
        return stream
    def _unhandled_exception(self):
        logger.error("An unexpected error has occurred", exc_info=True)
        if debug.is_on() or env.is_flagged("DEBUG"):
            self._exception(tb=True)
            if env.is_flagged("DEBUG"):
                post_mortem()
        self.error("An unexpected error has occurred - please contact customer support")

class InfiniboxShell(ShellMixin, Shell):
    def _on_start(self):
        for text, colors in CUSTOM_BANNER_PARTS:
            self.output(text, colors=colors, nl=False)

class InfiniboxExecutor(ShellMixin, Executor):
    pass
